@@ -10,7 +10,10 @@
 #' @param byrow Analogous to `byrow` in [matrix()][base::matrix]. If `FALSE` the
 #' plots will be filled in in column-major order
 #' @param widths,heights The relative widths and heights of each column and row
-#' in the grid. Will get repeated to match the dimensions of the grid.
+#' in the grid. Will get repeated to match the dimensions of the grid. The
+#' special value of `NA`/`-1null` will behave as `1null` unless a fixed aspect
+#' plot is inserted in which case it will allow the dimension to expand or
+#' contract to match the aspect ratio of the content
 #' @param guides A string specifying how guides should be treated in the layout.
 #' `'collect'` will collect guides below to the given nesting level, removing
 #' duplicates. `'keep'` will stop collection at this level and let guides be
@@ -24,6 +27,16 @@
 #' @param design Specification of the location of areas in the layout. Can either
 #' be specified as a text string or by concatenating calls to [area()] together.
 #' See the examples for further information on use.
+#' @param axes A string specifying how axes should be treated. `'keep'` will
+#' retain all axes in individual plots. `'collect'` will remove duplicated
+#' axes when placed in the same run of rows or columns of the layout.
+#' `'collect_x'` and `'collect_y'` will remove duplicated x-axes in the columns
+#' or duplicated y-axes in the rows respectively.
+#' @param axis_titles A string specifying how axis titltes should be treated.
+#' `'keep'` will retain all axis titles in individual plots. `'collect'` will
+#' remove duplicated titles in one direction and merge titles in the opposite
+#' direction. `'collect_x'` and `'collect_y'` control this for x-axis titles
+#' and y-axis titles respectively.
 #'
 #' @return A `plot_layout` object to be added to a `ggassmble` object
 #'
@@ -96,9 +109,15 @@
 
 plot_layout <- function(ncol = NULL, nrow = NULL, byrow = NULL, widths = NULL,
                         heights = NULL, guides = NULL, tag_level = NULL,
-                        design = NULL) {
+                        design = NULL, axes = NULL, axis_titles = axes) {
   if (!is.null(guides)) guides <- match.arg(guides, c('auto', 'collect', 'keep'))
   if (!is.null(tag_level)) tag_level <- match.arg(tag_level, c('keep', 'new'))
+  if (!is.null(axes)) axes <- match.arg(
+    axes, c('keep', 'collect', 'collect_x', 'collect_y')
+  )
+  if (!is.null(axis_titles)) collect_titles <- match.arg(
+    axis_titles, c('keep', 'collect', 'collect_x', 'collect_y')
+  )
   structure(list(
     ncol = ncol,
     nrow = nrow,
@@ -107,6 +126,8 @@ plot_layout <- function(ncol = NULL, nrow = NULL, byrow = NULL, widths = NULL,
     heights = heights,
     guides = guides,
     tag_level = tag_level,
+    axes = axes,
+    axis_titles = axis_titles,
     design = as_areas(design)
   ), class = 'plot_layout')
 }
@@ -180,10 +201,10 @@ area <- function(t, l, b = t, r = l) {
       r = rep_len(as.integer(r), len)
     )
     if (any(t > b)) {
-      stop('`t` must be less than `b`', call. = FALSE)
+      cli_abort('{.arg t} must be less than {.arg b}')
     }
     if (any(l > r)) {
-      stop('`l` must be less than `r`', call. = FALSE)
+      cli_abort('{.arg l} must be less than {.arg r}')
     }
   }
   class(one_area) <- 'patch_area'
@@ -231,7 +252,7 @@ as_areas <- function(x) {
   if (is.null(x)) return(NULL)
   if (is_area(x)) return(x)
   if (!is.character(x)) {
-    stop("Don't know how to convert ", class(x)[1], " into area positions", call. = FALSE)
+    cli_abort("Don't know how to convert {.cls {class(x)}} into area positions")
   }
   x <- strsplit(x, split = '\n')[[1]]
   x <- lapply(x, trimws)
@@ -240,7 +261,7 @@ as_areas <- function(x) {
   x <- lapply(x, function(x) strsplit(x, '')[[1]])
   ncols <- vapply(x, length, integer(1))
   if (length(unique(ncols)) != 1) {
-    stop("character layout must be rectangular", call. = FALSE)
+    cli_abort("character layout must be rectangular")
   }
   row <- rep(seq_along(x), each = ncols[1])
   col <- rep(seq_len(ncols[1]), length(x))
@@ -248,12 +269,12 @@ as_areas <- function(x) {
   area_names <- unique(sort(x))
   area_names[area_names == '#'] <- NA
   x <- match(x, area_names)
-  do.call(c, lapply(split(seq_along(x), x), function(i) {
+  exec(c, !!!lapply(split(seq_along(x), x), function(i) {
     if (is.na(x[i[1]])) return(area())
     area_rows <- range(row[i])
     area_cols <- range(col[i])
     if (!all(x[row >= area_rows[1] & row <= area_rows[2] & col >= area_cols[1] & col <= area_cols[2]] == x[i[1]])) {
-      stop('Patch areas must be rectangular', call. = FALSE)
+      cli_abort('Patch areas must be rectangular')
     }
     area(area_rows[1], area_cols[1], area_rows[2], area_cols[2])
   }))
@@ -273,7 +294,7 @@ c.patch_area <- function(..., recursive = FALSE) {
   if (length(all_areas) == 0) return(area())
 
   if (any(!vapply(all_areas, is_area, logical(1)))) {
-    stop('Areas can only be combined with each other', call. = FALSE)
+    cli_abort('Areas can only be combined with each other')
   }
   area <- all_areas[[1]]
   area$t <- unlist(lapply(all_areas, `[[`, 't'))
@@ -282,7 +303,10 @@ c.patch_area <- function(..., recursive = FALSE) {
   area$r <- unlist(lapply(all_areas, `[[`, 'r'))
   area
 }
-default_layout <- plot_layout(byrow = TRUE, widths = NA, heights = NA, guides = 'auto', tag_level = 'keep')
+default_layout <- plot_layout(
+  byrow = TRUE, widths = NA, heights = NA, guides = 'auto', tag_level = 'keep',
+  axes = 'keep', axis_titles = 'keep'
+)
 #' @importFrom utils modifyList
 #' @export
 ggplot_add.plot_layout <- function(object, plot, object_name) {
